@@ -198,6 +198,12 @@ AV *p5_sv_to_av(PerlInterpreter *my_perl, SV* sv) {
     return (AV *) SvRV(sv);
 }
 
+AV *p5_sv_to_av_inc(PerlInterpreter *my_perl, SV* sv) {
+    AV * const retval = (AV *) SvRV(sv);
+    SvREFCNT_inc((SV *)retval);
+    return retval;
+}
+
 HV *p5_sv_to_hv(PerlInterpreter *my_perl, SV* sv) {
     return (HV *) SvRV(sv);
 }
@@ -394,7 +400,8 @@ SV *p5_call_package_method(PerlInterpreter *my_perl, char *package, char *name,
 
         XPUSHs(newSVpv(package, 0));
         for (i = 0; i < len; i++) {
-            XPUSHs(sv_2mortal(args[i]));
+            if (args[i] != NULL) /* skip Nil which gets turned into NULL */
+                XPUSHs(sv_2mortal(args[i]));
         }
 
         PUTBACK;
@@ -453,11 +460,13 @@ SV *p5_call_method(PerlInterpreter *my_perl, char *package, SV *obj, I32 context
             if (len > 1) {
                 XPUSHs(package != NULL ? sv_2mortal(args[0]) : args[0]);
                 for (i = 1; i < len; i++) {
-                    XPUSHs(sv_2mortal(args[i]));
+                    if (args[i] != NULL) /* skip Nil which gets turned into NULL */
+                        XPUSHs(sv_2mortal(args[i]));
                 }
             }
             else if (len > 0)
-                XPUSHs(package != NULL ? sv_2mortal((SV*)args) : (SV*)args);
+                if (args != NULL) /* skip Nil which gets turned into NULL */
+                    XPUSHs(package != NULL ? sv_2mortal((SV*)args) : (SV*)args);
 
             PUTBACK;
 
@@ -515,7 +524,8 @@ SV *p5_call_function(PerlInterpreter *my_perl, char *name, int len, SV *args[],
         PUSHMARK(SP);
 
         for (i = 0; i < len; i++) {
-            XPUSHs(sv_2mortal(args[i]));
+            if (args[i] != NULL) /* skip Nil which gets turned into NULL */
+                XPUSHs(sv_2mortal(args[i]));
         }
 
         PUTBACK;
@@ -567,7 +577,8 @@ SV *p5_call_code_ref(PerlInterpreter *my_perl, SV *code_ref, int len, SV *args[]
         PUSHMARK(SP);
 
         for (i = 0; i < len; i++) {
-            XPUSHs(sv_2mortal(args[i]));
+            if (args[i] != NULL) /* skip Nil which gets turned into NULL */
+                XPUSHs(sv_2mortal(args[i]));
         }
 
         PUTBACK;
